@@ -14,18 +14,13 @@ from apgw.types import (ConnectionArgs, InsertAssignments, LimitOffset, Record,
 class DBBase:
     """A simple wrapper around asyncpg to alleviate some boilerplate."""
 
-    def __init__(
-        self,
-        *,
-        logger: Logger = getLogger("apgw"),
-        connect: Callable[..., Connection] = connect,
-    ) -> None:
+    def __init__(self) -> None:
         """Initializes the DB instance.
 
         :param logger: a Logger instance
         :param connection: method to connect to the database
         """
-        self.logger = logger
+        self.logger: Optional[Logger] = None
         self.connect = connect
         self._conn: Optional[Connection] = None
 
@@ -43,8 +38,7 @@ class DBBase:
     @property
     def should_auto_open(self) -> bool:
         """Returns True if the connection can and should be opened automatically."""
-        # return self._auto_open and not self.is_open
-        return True
+        return not self.is_open
 
     async def auto_open(self) -> bool:
         """Opens the database if it should auto open."""
@@ -53,6 +47,14 @@ class DBBase:
             return True
         return False
 
+    def set_logger(self, logger: Optional[Logger]) -> "DBBase":
+        self.logger = logger
+        return self
+
+    def set_connect(self, connect: Callable[..., Connection]) -> "DBBase":
+        self.connect = connect
+        return self
+
     async def get_connection_args(self) -> ConnectionArgs:
         raise NotImplementedError()
 
@@ -91,9 +93,10 @@ class DBBase:
 
     def _log(self, sql: str, params: List[Any] = None) -> None:
         """Log out sql and params."""
-        self.logger.info("SQL: %s", sql)
-        if params:
-            self.logger.info("PARAMS: %s", params)
+        if self.logger:
+            self.logger.info("SQL: %s", sql)
+            if params:
+                self.logger.info("PARAMS: %s", params)
 
     async def execute(self, sql: str, params: List[Any] = None):
         """This method passes through to self.conn.execute.
@@ -110,6 +113,21 @@ class DBBase:
         else:
             await self.conn.execute(sql)
 
+    async def executemany(self, sql: str, params: List[List[Any]] = None):
+        """This method passes through to self.conn.execute.
+
+        :param sql: the parameterized sql to execute
+        :param params: a list of params
+        """
+        self._log(sql, params)
+
+        await self.auto_open()
+
+        if params:
+            await self.conn.executemany(sql, params)
+        else:
+            await self.conn.executemany(sql)
+
     async def fetch(self, sql: Optional[str], params: List[Any] = None) -> List[Record]:
         """This method passes through to self.conn.fetch.
 
@@ -316,27 +334,21 @@ class DBBase:
 
 class DB(DBBase):
     """A DBBase implementation that has static connection args."""
-    def __init__(
-        self,
-        conn_args: ConnectionArgs,
-        *,
-        auto_open: bool = False,
-        logger: Logger = getLogger("apgw"),
-        connect: Callable[..., Connection] = connect,
-    ) -> None:
+
+    def __init__(self, conn_args: ConnectionArgs, *, auto_open: bool = False) -> None:
         """Initializes the DB instance.
 
         :param conn_args: the connection arguments
         :param auto_open: if the connection should automatically open
         :param logger: a Logger instance
         """
-        super().__init__(logger=logger, connect=connect)
+        super().__init__()
         self.conn_args = conn_args
         self._auto_open = auto_open
 
     @property
     def should_auto_open(self) -> bool:
-        return self._auto_open
+        return self._auto_open and not self.is_open
 
     async def get_connection_args(self) -> Connection:
         return self.conn_args
